@@ -3,10 +3,7 @@
 
 #include <stdint.h>
 
-// Doubles are still passed in GPRs, so the 'e' argument will be anyext as
-// GPRs are exhausted.
-
-// CHECK: define{{.*}} void @f_fpr_tracking(double noundef %a, double noundef %b, double noundef %c, double noundef %d, i8 noundef %e)
+// CHECK: define{{.*}} void @f_fpr_tracking(double noundef %a, double noundef %b, double noundef %c, double noundef %d, i8 noundef signext %e)
 void f_fpr_tracking(double a, double b, double c, double d, int8_t e) {}
 
 // Lowering for doubles is unnmodified, as 64 > FLEN.
@@ -17,17 +14,17 @@ struct double_s { double d; };
 void f_double_s_arg(struct double_s a) {}
 
 // CHECK: define{{.*}} i64 @f_ret_double_s()
-struct double_s f_ret_double_s() {
+struct double_s f_ret_double_s(void) {
   return (struct double_s){1.0};
 }
 
 struct double_double_s { double d; double e; };
 
-// CHECK: define{{.*}} void @f_double_double_s_arg(%struct.double_double_s* noundef %a)
+// CHECK: define{{.*}} void @f_double_double_s_arg(ptr noundef %a)
 void f_double_double_s_arg(struct double_double_s a) {}
 
-// CHECK: define{{.*}} void @f_ret_double_double_s(%struct.double_double_s* noalias sret(%struct.double_double_s) align 8 %agg.result)
-struct double_double_s f_ret_double_double_s() {
+// CHECK: define{{.*}} void @f_ret_double_double_s(ptr noalias sret(%struct.double_double_s) align 8 %agg.result)
+struct double_double_s f_ret_double_double_s(void) {
   return (struct double_double_s){1.0, 2.0};
 }
 
@@ -35,11 +32,11 @@ struct double_int8_s { double d; int64_t i; };
 
 struct int_double_s { int a; double b; };
 
-// CHECK: define{{.*}} void @f_int_double_s_arg(%struct.int_double_s* noundef %a)
+// CHECK: define{{.*}} void @f_int_double_s_arg(ptr noundef %a)
 void f_int_double_s_arg(struct int_double_s a) {}
 
-// CHECK: define{{.*}} void @f_ret_int_double_s(%struct.int_double_s* noalias sret(%struct.int_double_s) align 8 %agg.result)
-struct int_double_s f_ret_int_double_s() {
+// CHECK: define{{.*}} void @f_ret_int_double_s(ptr noalias sret(%struct.int_double_s) align 8 %agg.result)
+struct int_double_s f_ret_int_double_s(void) {
   return (struct int_double_s){1, 2.0};
 }
 
