@@ -10,20 +10,15 @@
 // as specified in section 18.5.5 of Fortran 2018.
 
 #include "flang/ISO_Fortran_binding.h"
+#include "ISO_Fortran_util.h"
+#include "terminator.h"
 #include "flang/Runtime/descriptor.h"
+#include "flang/Runtime/type-code.h"
 #include <cstdlib>
 
 namespace Fortran::ISO {
 extern "C" {
 
-static inline constexpr bool IsCharacterType(CFI_type_t ty) {
-  return ty == CFI_type_char || ty == CFI_type_char16_t ||
-      ty == CFI_type_char32_t;
-}
-static inline constexpr bool IsAssumedSize(const CFI_cdesc_t *dv) {
-  return dv->rank > 0 && dv->dim[dv->rank - 1].extent == -1;
-}
-
 void *CFI_address(
     const CFI_cdesc_t *descriptor, const CFI_index_t subscripts[]) {
   char *p{static_cast<char *>(descriptor->base_addr)};
@@ -72,7 +67,7 @@ int CFI_allocate(CFI_cdesc_t *descriptor, const CFI_index_t lower_bounds[],
     CFI_index_t lb{lower_bounds[j]};
     CFI_index_t ub{upper_bounds[j]};
     CFI_index_t extent{ub >= lb ? ub - lb + 1 : 0};
-    dim->lower_bound = lb;
+    dim->lower_bound = extent == 0 ? 1 : lb;
     dim->extent = extent;
     dim->sm = byteSize;
     byteSize *= extent;
@@ -106,184 +101,23 @@ int CFI_deallocate(CFI_cdesc_t *descriptor) {
   return CFI_SUCCESS;
 }
 
-static constexpr std::size_t MinElemLen(CFI_type_t type) {
-  std::size_t minElemLen{0};
-  switch (type) {
-  case CFI_type_signed_char:
-    minElemLen = sizeof(signed char);
-    break;
-  case CFI_type_short:
-    minElemLen = sizeof(short);
-    break;
-  case CFI_type_int:
-    minElemLen = sizeof(int);
-    break;
-  case CFI_type_long:
-    minElemLen = sizeof(long);
-    break;
-  case CFI_type_long_long:
-    minElemLen = sizeof(long long);
-    break;
-  case CFI_type_size_t:
-    minElemLen = sizeof(std::size_t);
-    break;
-  case CFI_type_int8_t:
-    minElemLen = sizeof(std::int8_t);
-    break;
-  case CFI_type_int16_t:
-    minElemLen = sizeof(std::int16_t);
-    break;
-  case CFI_type_int32_t:
-    minElemLen = sizeof(std::int32_t);
-    break;
-  case CFI_type_int64_t:
-    minElemLen = sizeof(std::int64_t);
-    break;
-  case CFI_type_int128_t:
-    minElemLen = 2 * sizeof(std::int64_t);
-    break;
-  case CFI_type_int_least8_t:
-    minElemLen = sizeof(std::int_least8_t);
-    break;
-  case CFI_type_int_least16_t:
-    minElemLen = sizeof(std::int_least16_t);
-    break;
-  case CFI_type_int_least32_t:
-    minElemLen = sizeof(std::int_least32_t);
-    break;
-  case CFI_type_int_least64_t:
-    minElemLen = sizeof(std::int_least64_t);
-    break;
-  case CFI_type_int_least128_t:
-    minElemLen = 2 * sizeof(std::int_least64_t);
-    break;
-  case CFI_type_int_fast8_t:
-    minElemLen = sizeof(std::int_fast8_t);
-    break;
-  case CFI_type_int_fast16_t:
-    minElemLen = sizeof(std::int_fast16_t);
-    break;
-  case CFI_type_int_fast32_t:
-    minElemLen = sizeof(std::int_fast32_t);
-    break;
-  case CFI_type_int_fast64_t:
-    minElemLen = sizeof(std::int_fast64_t);
-    break;
-  case CFI_type_intmax_t:
-    minElemLen = sizeof(std::intmax_t);
-    break;
-  case CFI_type_intptr_t:
-    minElemLen = sizeof(std::intptr_t);
-    break;
-  case CFI_type_ptrdiff_t:
-    minElemLen = sizeof(std::ptrdiff_t);
-    break;
-  case CFI_type_half_float:
-    minElemLen = 2;
-    break;
-  case CFI_type_bfloat:
-    minElemLen = 2;
-    break;
-  case CFI_type_float:
-    minElemLen = sizeof(float);
-    break;
-  case CFI_type_double:
-    minElemLen = sizeof(double);
-    break;
-  case CFI_type_extended_double:
-    minElemLen = 10;
-    break;
-  case CFI_type_long_double:
-    minElemLen = sizeof(long double);
-    break;
-  case CFI_type_float128:
-    minElemLen = 16;
-    break;
-  case CFI_type_half_float_Complex:
-    minElemLen = 2 * MinElemLen(CFI_type_half_float);
-    break;
-  case CFI_type_bfloat_Complex:
-    minElemLen = 2 * MinElemLen(CFI_type_bfloat);
-    break;
-  case CFI_type_float_Complex:
-    minElemLen = 2 * sizeof(float);
-    break;
-  case CFI_type_double_Complex:
-    minElemLen = 2 * sizeof(double);
-    break;
-  case CFI_type_extended_double_Complex:
-    minElemLen = 2 * MinElemLen(CFI_type_extended_double);
-    break;
-  case CFI_type_long_double_Complex:
-    minElemLen = 2 * sizeof(long double);
-    break;
-  case CFI_type_float128_Complex:
-    minElemLen = 2 * MinElemLen(CFI_type_float128);
-    break;
-  case CFI_type_Bool:
-    minElemLen = 1;
-    break;
-  case CFI_type_cptr:
-    minElemLen = sizeof(void *);
-    break;
-  case CFI_type_char16_t:
-    minElemLen = sizeof(char16_t);
-    break;
-  case CFI_type_char32_t:
-    minElemLen = sizeof(char32_t);
-    break;
-  }
-  return minElemLen;
-}
-
 int CFI_establish(CFI_cdesc_t *descriptor, void *base_addr,
     CFI_attribute_t attribute, CFI_type_t type, std::size_t elem_len,
     CFI_rank_t rank, const CFI_index_t extents[]) {
-  if (attribute != CFI_attribute_other && attribute != CFI_attribute_pointer &&
-      attribute != CFI_attribute_allocatable) {
-    return CFI_INVALID_ATTRIBUTE;
+  int cfiStatus{VerifyEstablishParameters(descriptor, base_addr, attribute,
+      type, elem_len, rank, extents, /*external=*/true)};
+  if (cfiStatus != CFI_SUCCESS) {
+    return cfiStatus;
   }
-  if (rank > CFI_MAX_RANK) {
-    return CFI_INVALID_RANK;
-  }
-  if (base_addr && attribute == CFI_attribute_allocatable) {
-    return CFI_ERROR_BASE_ADDR_NOT_NULL;
-  }
-  if (rank > 0 && base_addr && !extents) {
-    return CFI_INVALID_EXTENT;
-  }
-  if (type < CFI_type_signed_char || type > CFI_TYPE_LAST) {
-    return CFI_INVALID_TYPE;
-  }
-  if (!descriptor) {
-    return CFI_INVALID_DESCRIPTOR;
-  }
-  if (type == CFI_type_struct || type == CFI_type_other ||
-      IsCharacterType(type)) {
-    if (elem_len <= 0) {
-      return CFI_INVALID_ELEM_LEN;
-    }
-  } else {
+  if (type != CFI_type_struct && type != CFI_type_other &&
+      !IsCharacterType(type)) {
     elem_len = MinElemLen(type);
-    assert(elem_len > 0 && "Unknown element length for type");
   }
-  descriptor->base_addr = base_addr;
-  descriptor->elem_len = elem_len;
-  descriptor->version = CFI_VERSION;
-  descriptor->rank = rank;
-  descriptor->type = type;
-  descriptor->attribute = attribute;
-  descriptor->f18Addendum = 0;
-  std::size_t byteSize{elem_len};
-  constexpr std::size_t lower_bound{0};
-  if (base_addr) {
-    for (std::size_t j{0}; j < rank; ++j) {
-      descriptor->dim[j].lower_bound = lower_bound;
-      descriptor->dim[j].extent = extents[j];
-      descriptor->dim[j].sm = byteSize;
-      byteSize *= extents[j];
-    }
+  if (elem_len <= 0) {
+    return CFI_INVALID_ELEM_LEN;
   }
+  EstablishDescriptor(
+      descriptor, base_addr, attribute, type, elem_len, rank, extents);
   return CFI_SUCCESS;
 }
 
@@ -361,8 +195,10 @@ int CFI_section(CFI_cdesc_t *result, const CFI_cdesc_t *source,
   resRank = 0;
   for (int j{0}; j < source->rank; ++j) {
     if (actualStride[j] != 0) {
-      result->dim[resRank].lower_bound = 0;
       result->dim[resRank].extent = extent[j];
+      result->dim[resRank].lower_bound = extent[j] == 0 ? 1
+          : lower_bounds                                ? lower_bounds[j]
+                         : source->dim[j].lower_bound;
       result->dim[resRank].sm = actualStride[j] * source->dim[j].sm;
       ++resRank;
     }
@@ -437,10 +273,12 @@ int CFI_setpointer(CFI_cdesc_t *result, const CFI_cdesc_t *source,
   result->base_addr = source->base_addr;
   if (source->base_addr) {
     for (int j{0}; j < result->rank; ++j) {
-      result->dim[j].extent = source->dim[j].extent;
+      CFI_index_t extent{source->dim[j].extent};
+      result->dim[j].extent = extent;
       result->dim[j].sm = source->dim[j].sm;
-      result->dim[j].lower_bound =
-          copySrcLB ? source->dim[j].lower_bound : lower_bounds[j];
+      result->dim[j].lower_bound = extent == 0 ? 1
+          : copySrcLB                          ? source->dim[j].lower_bound
+                                               : lower_bounds[j];
     }
   }
   return CFI_SUCCESS;
