@@ -298,3 +298,39 @@ contains
     real :: x
   end
 end
+
+module m8
+  interface
+    pure elemental module subroutine s1
+    end subroutine
+  end interface
+end module
+
+submodule(m8) sm8
+ contains
+  !Ensure no spurious error about mismatching attributes
+  module procedure s1
+  end procedure
+end submodule
+
+module m9
+  interface
+    module subroutine sub1(s)
+      character(len=0) s
+    end subroutine
+    module subroutine sub2(s)
+      character(len=0) s
+    end subroutine
+  end interface
+end module
+
+submodule(m9) sm1
+ contains
+  module subroutine sub1(s)
+    character(len=-1) s ! ok
+  end subroutine
+  module subroutine sub2(s)
+    !ERROR: Dummy argument 's' has type CHARACTER(KIND=1,LEN=1_8); the corresponding argument in the interface body has type CHARACTER(KIND=1,LEN=0_8)
+    character(len=1) s
+  end subroutine
+end submodule
