@@ -44,7 +44,7 @@
 #include "filesystem_include.h"
 #include "filesystem_test_helper.h"
 
-int main() {
+int main(int, char**) {
   scoped_test_env env;
   fs::path const tmpdir = env.create_dir("mydir");
   fs::path const victim_del_path = tmpdir / "victim_del";
