@@ -10,7 +10,7 @@
 
 // class deque
 
-// bool empty() const noexcept;
+// bool empty() const noexcept; // constexpr since C++20
 
 // UNSUPPORTED: c++03, c++11, c++14, c++17
 
@@ -18,10 +18,19 @@
 
 #include "test_macros.h"
 
+TEST_CONSTEXPR_CXX20 bool test() {
+  std::string c;
+  c.empty(); // expected-warning {{ignoring return value of function declared with 'nodiscard' attribute}}
+
+  return true;
+}
+
 int main(int, char**)
 {
-    std::string c;
-    c.empty(); // expected-warning {{ignoring return value of function declared with 'nodiscard' attribute}}
+  test();
+#if TEST_STD_VER > 17
+  static_assert(test());
+#endif
 
-    return 0;
+  return 0;
 }
