@@ -6,7 +6,7 @@
 //
 //===----------------------------------------------------------------------===//
 //
-// UNSUPPORTED: libcpp-has-no-threads
+// UNSUPPORTED: no-threads
 // UNSUPPORTED: c++03, c++11
 
 // This test requires the dylib support introduced in D68480, which shipped in macOS 11.0.
@@ -24,7 +24,7 @@
 int main(int, char**)
 {
   int x = 0;
-  auto comp = [&]() { x += 1; };
+  auto comp = [&]() noexcept { x += 1; };
   std::barrier<decltype(comp)> b(2, comp);
 
   std::thread t = support::make_test_thread([&](){
